/*
ID: espr1t
TASK: End-to-End
KEYWORDS: Medium, Graph, Dijkstra
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <queue>
#include <ctime>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 505;
const int INF = 1000000001;
const double TIME_LIMIT = 1.3;

struct State {
    int row, col, mask, price;
    State(int row_ = 0, int col_ = 0, int mask_ = 0, int price_ = 0) :
        row(row_), col(col_), mask(mask_), price(price_) {}
    bool operator < (const State& r) const {
        return price != r.price ? price > r.price :
            (row != r.row ? row < r.row : (col != r.col ? col < r.col : mask < r.mask));
    }
};

int n, m;
char a[MAX][MAX];

int distL[MAX][MAX];
int distR[MAX][MAX];
int distA[4][MAX][MAX];

bool vis[4][MAX][MAX];
priority_queue <State> q;

int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

int getCost(int row, int col, int digit) {
    return abs(a[row][col] - '0' - digit);
}

void dijkstra(int digit, int dist[MAX][MAX], int startCol) {
    // Note that the price to a cell *does not* include the cost of the last cell
    // (for convenience later on when combining the paths).
    memset(vis, 0, sizeof(vis));
    memset(dist, 63, sizeof(dist[0][0]) * MAX * MAX);
    while (!q.empty())
        q.pop();

    for (int row = 0; row < n; row++) {
        dist[row][startCol] = 0;
        q.push(State(row, startCol, 0, 0));
    }
    while (!q.empty()) {
        State state = q.top(); q.pop();
        if (vis[state.mask][state.row][state.col])
            continue;
        vis[state.mask][state.row][state.col] = true;
        int nextPrice = state.price + getCost(state.row, state.col, digit);
        for (int d = 0; d < 4; d++) {
            int nrow = state.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
            int ncol = state.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
            if (dist[nrow][ncol] > nextPrice) {
                dist[nrow][ncol] = nextPrice;
                q.push(State(nrow, ncol, 0, nextPrice));
            }
        }
    }
}

int bestPrice = INF;
int eval(int digit) {
    memset(vis, 0, sizeof(vis));
    memset(distA, 63, sizeof(distA));
    while (!q.empty())
        q.pop();
    
    for (int col = 0; col < m; col++) {
        int row = 0, mask = 0;
        distA[mask][row][col] = getCost(row, col, digit);
        q.push(State(row, col, mask, distA[mask][row][col]));
    }
    while (!q.empty()) {
        State state = q.top(); q.pop();
        if (vis[state.mask][state.row][state.col])
            continue;
        vis[state.mask][state.row][state.col] = true;

        if (state.row == n - 1 && state.mask == 3)
            return state.price;

        // Add connection to one of the sides
        for (int side = 1; side <= 2; side++) if (!(state.mask & side)) {
            int nextPrice = state.price +
                (side == 1 ? distL[state.row][state.col] : distR[state.row][state.col]);
            if (distA[state.mask | side][state.row][state.col] > nextPrice) {
                distA[state.mask | side][state.row][state.col] = nextPrice;
                q.push(State(state.row, state.col, state.mask | side, nextPrice));
            }
        }

        // Continue to another cell
        for (int d = 0; d < 4; d++) {
            int nrow = state.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
            int ncol = state.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
            int nextPrice = state.price + getCost(nrow, ncol, digit);
            if (nextPrice < bestPrice) {
                if (distA[state.mask][nrow][ncol] > nextPrice && bestPrice > nextPrice) {
                    distA[state.mask][nrow][ncol] = nextPrice;
                    q.push(State(nrow, ncol, state.mask, nextPrice));
                }
            }
        }
    }
    return INF;
}

int solve() {
    unsigned sTime = clock();
    int digits[10] = {4, 5, 3, 6, 2, 7, 1, 8, 0, 9};
    double lastTime = 0.0;
    for (int d = 0; d < 10; d++) {
        unsigned dTime = clock();
        dijkstra(digits[d], distL, 0);
        dijkstra(digits[d], distR, m - 1);
        bestPrice = min(bestPrice, eval(digits[d]));
        lastTime = (double)(clock() - dTime) / CLOCKS_PER_SEC;
        if ((double)(clock() - sTime) / CLOCKS_PER_SEC > TIME_LIMIT - lastTime * 2)
            break;
    }
    return bestPrice;
}

int main(void) {
    // in = fopen("EndToEnd.in", "rt");
    
    fscanf(in, "%d %d", &n, &m);
    for (int row = 0; row < n; row++)
        fscanf(in, "%s", a[row]);
    fprintf(out, "%d\n", solve());
    
    return 0;
}
