/*
ID: espr1t
TASK: End-to-End
KEYWORDS: Medium, Graph, Dijkstra
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <queue>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 505;
const int INF = 1000000001;
const int LIM = (MAX + MAX) * 9;

struct Cell {
    int row, col;
    Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
};

int n, m;
char a[MAX][MAX];

queue <Cell> QLR[LIM];
bool vis[MAX][MAX];
int distL[MAX][MAX];
int distR[MAX][MAX];

queue <Cell> QUD[4][LIM];
bool visAll[4][MAX][MAX];
int distAll[4][MAX][MAX];

int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

int getCost(int row, int col, int digit) {
    return abs(a[row][col] - '0' - digit);
}

// This modified Dijkstra's algorithm implementation exploits the fact that all
// edges have weights between 0 and 9, and instead of using priority queue, just
// uses standard queues and iterates all possible prices in increasing order.
// As the max price is (MAX + MAX) * 9 this is faster than a PQ for large tests.
void dijkstra(int digit, int dist[MAX][MAX], int startCol) {
    // Note that the price to a cell *does not* include the cost of the last cell
    // (for convenience later on when combining the paths).
    memset(vis, 0, sizeof(vis));
    memset(dist, 63, sizeof(dist[0][0]) * MAX * MAX);
    for (int row = 0; row < n; row++) {
        dist[row][startCol] = 0;
        QLR[0].push(Cell(row, startCol));
    }
    for (int price = 0; price < LIM; price++) {
        while (!QLR[price].empty()) {
            Cell cell = QLR[price].front(); QLR[price].pop();
            if (vis[cell.row][cell.col])
                continue;
            vis[cell.row][cell.col] = true;
            int nextPrice = price + getCost(cell.row, cell.col, digit);
            for (int d = 0; d < 4; d++) {
                int nrow = cell.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
                int ncol = cell.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
                if (dist[nrow][ncol] > nextPrice) {
                    dist[nrow][ncol] = nextPrice;
                    QLR[nextPrice].push(Cell(nrow, ncol));
                }
            }
        }
    }
}

int eval(int digit) {
    memset(visAll, 0, sizeof(visAll));
    memset(distAll, 63, sizeof(distAll));
    for (int mask = 0; mask < 4; mask++) {
        for (int price = 0; price < LIM; price++) {
            while (!QUD[mask][price].empty()) {
                QUD[mask][price].pop();
            }
        }
    }
    
    for (int col = 0; col < m; col++) {
        distAll[0][0][col] = getCost(0, col, digit);
        QUD[0][distAll[0][0][col]].push(Cell(0, col));
    }
    for (int price = 0; price < LIM; price++) {
        for (int mask = 0; mask < 4; mask++) {
            while (!QUD[mask][price].empty()) {
                Cell cell = QUD[mask][price].front(); QUD[mask][price].pop();
                if (visAll[mask][cell.row][cell.col])
                    continue;
                visAll[mask][cell.row][cell.col] = true;

                if (cell.row == n - 1 && mask == 3)
                    return price;

                // Add connection to one of the sides
                for (int side = 1; side <= 2; side++) if (!(mask & side)) {
                    int nextPrice = price + (side == 1 ? distL[cell.row][cell.col] : distR[cell.row][cell.col]);
                    if (distAll[mask | side][cell.row][cell.col] > nextPrice) {
                        distAll[mask | side][cell.row][cell.col] = nextPrice;
                        QUD[mask | side][nextPrice].push(cell);
                    }
                }

                // Continue to another cell
                for (int d = 0; d < 4; d++) {
                    int nrow = cell.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
                    int ncol = cell.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
                    int nextPrice = price + getCost(nrow, ncol, digit);
                    if (distAll[mask][nrow][ncol] > nextPrice) {
                        distAll[mask][nrow][ncol] = nextPrice;
                        QUD[mask][nextPrice].push(Cell(nrow, ncol));
                    }
                }
            }
        }
    }
    return INF;
}

int solve() {
    int ans = INF;
    for (int digit = 0; digit < 10; digit++) {
        dijkstra(digit, distL, 0);
        dijkstra(digit, distR, m - 1);
        ans = min(ans, eval(digit));
    }
    return ans;
}

int main(void) {
    // in = fopen("EndToEnd.in", "rt");
    
    fscanf(in, "%d %d", &n, &m);
    for (int row = 0; row < n; row++)
        fscanf(in, "%s", a[row]);
    fprintf(out, "%d\n", solve());
    
    return 0;
}
