/*
ID: espr1t
TASK: End-to-End
KEYWORDS: Medium, Graph, Dijkstra
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <queue>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 505;
const int INF = 1000000001;
const int LIM = (MAX + MAX) * 9;

struct Cell {
    int row, col;
    Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
};

int n, m;
char a[MAX][MAX];

queue <Cell> q[LIM];
bool vis[MAX][MAX];
int distL[MAX][MAX];
int distR[MAX][MAX];
int distT[MAX][MAX];
int distB[MAX][MAX];

int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

int getCost(int row, int col, int digit) {
    return abs(a[row][col] - '0' - digit);
}

// This modified Dijkstra's algorithm implementation exploits the fact that all
// edges have weights between 0 and 9, and instead of using priority queue, just
// uses standard queues and iterates all possible prices in increasing order.
// As the max price is (MAX + MAX) * 9 this is faster than a PQ for large tests.
void dijkstra(int digit, int dist[MAX][MAX], int startRow, int startCol) {
    // Note that the price to a cell *does not* include the cost of the last cell
    // (for convenience later on when combining the paths).
    memset(vis, 0, sizeof(vis));
    memset(dist, 63, sizeof(dist[0][0]) * MAX * MAX);
    for (int row = 0; row < n; row++) {
        for (int col = 0; col < m; col++) {
            if (startRow == row || startCol == col) {
                dist[row][col] = 0;
                q[0].push(Cell(row, col));
            }
        }
    }

    for (int price = 0; price < LIM; price++) {
        while (!q[price].empty()) {
            Cell cell = q[price].front(); q[price].pop();
            if (vis[cell.row][cell.col])
                continue;
            vis[cell.row][cell.col] = true;
            int nextPrice = price + getCost(cell.row, cell.col, digit);
            for (int d = 0; d < 4; d++) {
                int nrow = cell.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
                int ncol = cell.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
                if (dist[nrow][ncol] > nextPrice) {
                    dist[nrow][ncol] = nextPrice;
                    q[nextPrice].push(Cell(nrow, ncol));
                }
            }
        }
    }
}

int solve() {
    int ans = INF;
    for (int digit = 0; digit < 10; digit++) {
        dijkstra(digit, distL, -1, 0);
        dijkstra(digit, distR, -1, m - 1);
        dijkstra(digit, distT, 0, -1);
        dijkstra(digit, distB, n - 1, -1);
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++) {
                ans = min(ans, getCost(row, col, digit) +
                    distL[row][col] + distR[row][col] + distT[row][col] + distB[row][col]);
            }
        }
    }
    return ans;
}

int main(void) {
    // in = fopen("EndToEnd.in", "rt");
    
    fscanf(in, "%d %d", &n, &m);
    for (int row = 0; row < n; row++)
        fscanf(in, "%s", a[row]);
    fprintf(out, "%d\n", solve());
    
    return 0;
}
