/*
ID: espr1t
TASK: End-to-End
KEYWORDS: Medium, Graph, Dijkstra, DP
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <queue>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

const int MAX = 505;
const int INF = 1000000001;

struct Cell {
    int row, col;
    Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
};

int n, m;
char a[MAX][MAX];

queue <Cell> QLR[10];
bool vis[MAX][MAX];
short distL[MAX][MAX];
short distR[MAX][MAX];

int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

int getCost(int row, int col, int digit) {
    return abs(a[row][col] - '0' - digit);
}

// This modified Dijkstra's algorithm implementation exploits the fact that all
// edges have weights between 0 and 9, and instead of using priority queue, just
// uses standard queues and iterates all possible prices in increasing order.
// As the max price is (MAX + MAX) * 9 this is faster than a PQ for large tests.
void dijkstra(int digit, short dist[MAX][MAX], int startCol) {
    // Note that the price to a cell *does not* include the cost of the last cell
    // (for convenience later on when combining the paths).
    memset(vis, 0, sizeof(vis));
    memset(dist, 63, sizeof(dist[0][0]) * MAX * MAX);
    for (int row = 0; row < n; row++) {
        dist[row][startCol] = 0;
        QLR[0].push(Cell(row, startCol));
    }
    for (int price = 0, last = 0; price - last < 10; price++) {
        while (!QLR[price % 10].empty()) {
            Cell cell = QLR[price % 10].front(); QLR[price % 10].pop();
            if (vis[cell.row][cell.col])
                continue;
            vis[cell.row][cell.col] = true;
            last = price;
            short nextPrice = price + getCost(cell.row, cell.col, digit);
            for (int d = 0; d < 4; d++) {
                int nrow = cell.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
                int ncol = cell.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
                if (dist[nrow][ncol] > nextPrice) {
                    dist[nrow][ncol] = nextPrice;
                    QLR[nextPrice % 10].push(Cell(nrow, ncol));
                }
            }
        }
    }
}

short dyn[MAX][MAX][4][3];
short recurse(int row, int col, int mask, int last, int digit) {
    if (row == n - 1 && mask == 3)
        return 0;
    if (dyn[row][col][mask][last] != -1)
        return dyn[row][col][mask][last];
    
    int ans = INF;
    // Connect to left side
    if (!(mask & 1)) {
        ans = min(ans, recurse(row, col, mask | 1, last, digit) + distL[row][col]);
    }
    // Connect to right side
    if (!(mask & 2)) {
        ans = min(ans, recurse(row, col, mask | 2, last, digit) + distR[row][col]);
    }
    // Go down
    if (row + 1 < n) {
        ans = min(ans, recurse(row + 1, col, mask, 0, digit) + getCost(row + 1, col, digit));
    }
    // Go left
    if (col > 0 && last != 2) {
        ans = min(ans, recurse(row, col - 1, mask, 1, digit) + getCost(row, col - 1, digit));
    }
    // Go right
    if (col + 1 < m && last != 1) {
        ans = min(ans, recurse(row, col + 1, mask, 2, digit) + getCost(row, col + 1, digit));
    }
    return dyn[row][col][mask][last] = ans;
}

int eval(int digit) {
    memset(dyn, -1, sizeof(dyn));
    int ans = INF;
    for (int col = 0; col < m; col++) {
        ans = min(ans, recurse(0, col, 0, 0, digit) + getCost(0, col, digit));
    }
    return ans;
}

int solve() {
    int ans = INF;
    for (int digit = 0; digit < 10; digit++) {
        dijkstra(digit, distL, 0);
        dijkstra(digit, distR, m - 1);
        ans = min(ans, eval(digit));
    }
    return ans;
}

int main(void) {
    // in = fopen("EndToEnd.in", "rt");
    
    fscanf(in, "%d %d", &n, &m);
    for (int row = 0; row < n; row++)
        fscanf(in, "%s", a[row]);
    fprintf(out, "%d\n", solve());
    
    return 0;
}
