/*
ID: espr1t
TASK: End-to-End
KEYWORDS: Testgen
*/

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <queue>
#include <random>
#include <ctime>
#include <vector>
#include <cassert>

using namespace std;
FILE* in = stdin; FILE* out = stdout;

mt19937 mt;
int randd() {
    int ret = mt();
    return ret < 0 ? -ret : ret;
}

struct Test {
    int N, M, S, A, B, MOD;
    Test(int N_ = 0, int M_ = 0, int S_ = 0, int A_ = 0, int B_ = 0, int MOD_ = 0) :
        N(N_), M(M_), S(S_), A(A_), B(B_), MOD(MOD_) {}
};
vector <Test> tests;
int cntLow, cntSingle, cntNone;

const int MAX = 505;
const int INF = 1000000001;
const int LIM = (MAX + MAX) * 20;

struct Cell {
    int row, col;
    Cell(int row_ = 0, int col_ = 0) : row(row_), col(col_) {}
};

int n, m;
char a[MAX][MAX];

queue <Cell> QLR[LIM];
bool vis[MAX][MAX];
int distL[MAX][MAX];
int distR[MAX][MAX];
int distT[MAX][MAX];
int distB[MAX][MAX];

queue <Cell> QUD[4][LIM];
bool visAll[4][MAX][MAX];
int distAll[4][MAX][MAX];

int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

int getCost(int row, int col, int digit) {
    return abs(a[row][col] - '0' - digit);
}

// This modified Dijkstra's algorithm implementation exploits the fact that all
// edges have weights between 0 and 9, and instead of using priority queue, just
// uses standard queues and iterates all possible prices in increasing order.
// As the max price is (MAX + MAX) * 9 this is faster than a PQ for large tests.
void dijkstra(int digit, int dist[MAX][MAX], int startRow, int startCol) {
    // Note that the price to a cell *does not* include the cost of the last cell
    // (for convenience later on when combining the paths).
    memset(vis, 0, sizeof(vis));
    memset(dist, 63, sizeof(dist[0][0]) * MAX * MAX);
    for (int row = 0; row < n; row++) {
        for (int col = 0; col < m; col++) {
            if (startRow == row || startCol == col) {
                dist[row][col] = 0;
                QLR[0].push(Cell(row, col));
            }
        }
    }

    for (int price = 0; price < LIM; price++) {
        while (!QLR[price].empty()) {
            Cell cell = QLR[price].front(); QLR[price].pop();
            if (vis[cell.row][cell.col])
                continue;
            vis[cell.row][cell.col] = true;
            int nextPrice = price + getCost(cell.row, cell.col, digit);
            for (int d = 0; d < 4; d++) {
                int nrow = cell.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
                int ncol = cell.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
                if (dist[nrow][ncol] > nextPrice) {
                    dist[nrow][ncol] = nextPrice;
                    QLR[nextPrice].push(Cell(nrow, ncol));
                }
            }
        }
    }
}
int prv[4][MAX][MAX];

int eval(int digit) {
    memset(visAll, 0, sizeof(visAll));
    memset(distAll, 63, sizeof(distAll));
    for (int mask = 0; mask < 4; mask++) {
        for (int price = 0; price < LIM; price++) {
            while (!QUD[mask][price].empty()) {
                QUD[mask][price].pop();
            }
        }
    }
    
    for (int col = 0; col < m; col++) {
        distAll[0][0][col] = getCost(0, col, digit);
        QUD[0][distAll[0][0][col]].push(Cell(0, col));
    }
    for (int price = 0; price < LIM; price++) {
        for (int mask = 0; mask < 4; mask++) {
            while (!QUD[mask][price].empty()) {
                Cell cell = QUD[mask][price].front(); QUD[mask][price].pop();
                if (visAll[mask][cell.row][cell.col])
                    continue;
                visAll[mask][cell.row][cell.col] = true;

                if (cell.row == n - 1 && mask == 3)
                    return price;

                // Add connection to one of the sides
                for (int side = 1; side <= 2; side++) if (!(mask & side)) {
                    int nextPrice = price + (side == 1 ? distL[cell.row][cell.col] : distR[cell.row][cell.col]);
                    if (distAll[mask | side][cell.row][cell.col] > nextPrice) {
                        distAll[mask | side][cell.row][cell.col] = nextPrice;
                        QUD[mask | side][nextPrice].push(cell);
                        prv[mask | side][cell.row][cell.col] = -side;
                    }
                }

                // Continue to another cell
                for (int d = 0; d < 4; d++) {
                    int nrow = cell.row + dir[d][0]; if (nrow < 0 || nrow >= n) continue;
                    int ncol = cell.col + dir[d][1]; if (ncol < 0 || ncol >= m) continue;
                    int nextPrice = price + getCost(nrow, ncol, digit);
                    if (distAll[mask][nrow][ncol] > nextPrice) {
                        distAll[mask][nrow][ncol] = nextPrice;
                        QUD[mask][nextPrice].push(Cell(nrow, ncol));
                        prv[mask][nrow][ncol] = cell.row * m + cell.col;
                    }
                }
            }
        }
    }
    return INF;
}

int evalSingle(int digit) {
    int ans = INF;
    for (int row = 0; row < n; row++) {
        for (int col = 0; col < m; col++) {
            ans = min(ans, getCost(row, col, digit) +
                distL[row][col] + distR[row][col] + distT[row][col] + distB[row][col]);
        }
    }
    return ans;
}

void restore(int mask, int row, int col) {
    fprintf(stderr, "%d %d (mask %d)\n", row, col, mask);
    if (row == 0)
        return;
    if (prv[mask][row][col] < 0) {
        restore(mask ^ -prv[mask][row][col], row, col);
    } else {
        int nrow = prv[mask][row][col] / m;
        int ncol = prv[mask][row][col] % m;
        restore(mask, nrow, ncol);
    }
}

int bestDigit, bestCount;
int solve() {
    bestDigit = -1, bestCount = 0;

    int ans = INF, ansSingle = INF;
    unsigned sTime = clock();
    for (int digit = 0; digit < 10; digit++) {
        dijkstra(digit, distL, -1, 0);
        dijkstra(digit, distR, -1, m - 1);
        dijkstra(digit, distT, 0, -1);
        dijkstra(digit, distB, n - 1, -1);
        int cur = eval(digit);
        int single = evalSingle(digit);
//        fprintf(stderr, "With digit %d: %d (%.3lfs total time)\n",
//            digit, cur, (double)(clock() - sTime) / CLOCKS_PER_SEC);
        if (ans > cur)
            bestDigit = digit, bestCount = 1;
        if (ans == cur)
            bestDigit = digit, bestCount++;
        ans = min(ans, cur);
        ansSingle = min(ansSingle, single);
        
//        for (int col = 0; col < m; col++) {
//            if (visAll[3][n - 1][col]) {
//                fprintf(stderr, "Path:\n");
//                restore(3, n - 1, col);
//                break;
//            }
//        }
    }
    fprintf(stderr, "  >> best digit: %d (%d options).\n", bestDigit, bestCount);
    if (n <= 250 && m <= 250) {
        cntLow++;
        fprintf(stderr, "    -- constraints on N and M are below 100.\n");
    }
    if (ans == ansSingle) {
        cntSingle++;
        fprintf(stderr, "    -- can be solved using single meeting point.\n");
    }
    if ((n > 250 || m > 250) && ans != ansSingle) {
        cntNone++;
    }
    fprintf(stderr, "  >> execution time: %.3lfs\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);
    return ans;
}

void generate(int N, int M, int S, int A, int B, int MOD, bool addTest=true) {
    if (addTest) {
        tests.push_back(Test(N, M, S, A, B, MOD));
    }

    memset(a, 0, sizeof(a));
    n = N, m = M;
    for (int row = 0; row < n; row++) {
        for (int col = 0; col < m; col++) {
            a[row][col] = S % 10 + '0';
            S = ((long long)S * A + B) % MOD;
        }
    }
//    fprintf(stderr, "Generated board:\n");
//    fprintf(stderr, "%d %d\n", n, m);
//    for (int row = 0; row < n; row++)
//        fprintf(stderr, "%s\n", a[row]);

//    solve();
}

void sampleInput1() {
    generate(4, 7, 42, 6661337, 13, 1000000009);
}

void sampleInput2() {
    generate(1, 10, 4, 2, 0, 6);
}

void sampleInput3() {
    generate(8, 8, 1, 2, 2, 3);
}

void sampleInput4() {
    generate(123, 321, 42, 13, 17, 987654321);
}

void maxTest1() {
    // Only optimal solution is digit 0.
    generate(500, 500, 354313780, 209663994, 444925560, 538788865);
}

void maxTest2() {
    // Only optimal solution is digit 1.
    generate(500, 500, 252943446, 6354254, 387649386, 411474095);
}

void maxTest3() {
    // Only optimal solution is digit 2.
    generate(500, 500, 42, 48271, 0, 536870912);
}

void maxTest4() {
    // Only optimal solution is digit 2.
    generate(500, 500, 15372, 1341, 666, 536870912);
}

void maxTest5() {
    // Only optimal solution is digit 4.
    generate(500, 500, 42, 426661337, 133742666, 1000000007);
}

void maxTest6() {
    // Only optimal solution is digit 5.
    generate(500, 500, 42, 1337, 13, 987654321);
}

void maxTest7() {
    // Only optimal solution is digit 7.
    generate(500, 500, 3344769, 1341, 666, 536870912);
}

void maxTest8() {
    // Only optimal solution is digit 9.
    generate(500, 500, 169695579, 142397840, 80099264, 247609285);
}

void maxTest9() {
    generate(500, 500, 123, 456, 789, 1000000007);
}

void maxTest10() {
    generate(500, 500, 666, 99999989, 100000039, 1000000009);
}

bool isPrime(int num) {
    if (num < 2) return false;
    for (int d = 2; d * d <= num; d++)
        if (num % d == 0) return false;
    return true;
}

void randomTest(int N, int M, bool primeMod=false) {
    int MOD = randd() % 900000000 + 100000000;
    if (primeMod) {
        while (!isPrime(MOD)) {
            MOD = randd() % 900000000 + 100000000;
        }
    }
    int S = randd() % MOD;
    int A = randd() % MOD;
    int B = randd() % MOD;
    generate(N, M, S, A, B, MOD);
}

void findSeed() {
    while (true) {
        int MOD = randd() % 900000000 + 100000000;
        int A = randd() % MOD;
        int B = randd() % MOD;
        int seed = randd() % MOD;
        generate(500, 500, seed, A, B, MOD);
        solve();
        if (bestCount <= 1 && bestDigit == 0) {
            fprintf(stderr, "seed = %d, A = %d, B = %d, MOD = %d\n", seed, A, B, MOD);
            break;
        }
    }
}

void sanityCheck(const Test& test) {
    assert(test.N >= 1 && test.N <= 500);
    assert(test.M >= 1 && test.M <= 500);
    assert(test.MOD >= 1 && test.MOD <= 1000000009);
    assert(test.S >= 0 && test.S < test.MOD);
    assert(test.A >= 0 && test.A < test.MOD);
    assert(test.B >= 0 && test.B < test.MOD);
}

void printTests() {
    fprintf(stderr, "Number of tests: %d\n", (int)tests.size());
    cntLow = cntSingle = cntNone = 0;
    
    for (int t = 0; t < (int)tests.size(); t++) {
        fprintf(stderr, "Currently printing testcase %d...\n", t);
        
        sanityCheck(tests[t]);
        generate(tests[t].N, tests[t].M, tests[t].S, tests[t].A, tests[t].B, tests[t].MOD, false);
        
        char inpFile[32], solFile[32];
        sprintf(inpFile, "EndToEnd.%02d.in", t);
        sprintf(solFile, "EndToEnd.%02d.sol", t);
        
        FILE* inp = fopen(inpFile, "wt");
        fprintf(inp, "%d %d\n", n, m);
        for (int row = 0; row < n; row++)
            fprintf(inp, "%s\n", a[row]);
        fclose(inp);
        
        int ans = solve();
        
        FILE* sol = fopen(solFile, "wt");
        fprintf(sol, "%d\n", ans);
        fclose(sol);
    }
    fprintf(stderr, "Tests that have low constraints: %d\n", cntLow);
    fprintf(stderr, "Tests that can be solved from single cell: %d\n", cntSingle);
    fprintf(stderr, "Tests that have no limitations: %d\n", cntNone);
}

int main(void) {
    mt.seed(42);

    // findSeed();

    // Samples
    sampleInput1(); // Low, single (sample, ignored)
    sampleInput2(); // Low, single
    sampleInput3(); // Low, single
    
    // Low constraints, single meeting point
    mt.seed(1000);
    randomTest(1, 1, true); // Low, single
    mt.seed(2000);
    randomTest(20, 1, true); // Low, single
    mt.seed(3007);
    randomTest(42, 42, false); // Low, single
    mt.seed(4002);
    randomTest(66, 66, false); // Low, single
    mt.seed(5004);
    randomTest(100, 100, false); // Low, single
    mt.seed(6000);
    randomTest(150, 150, false); // Low, single
    mt.seed(7023);
    randomTest(250, 250, false); // Low, single
    mt.seed(8016);
    randomTest(250, 250); // Low, single

//    for (int s = 8000; s < 8100; s++) { 
//        fprintf(stderr, "seed = %d\n", s);
//        mt.seed(s);
//        randomTest(250, 250, false);
//    }

    // Low constraints
    mt.seed(1);
    randomTest(13, 17, true); // Low
    mt.seed(2);
    randomTest(20, 20, false); // Low
    mt.seed(3);
    randomTest(50, 50, true); // Low
    mt.seed(4);
    randomTest(88, 75, false); // Low
    mt.seed(5);
    randomTest(101, 133, true); // Low
    mt.seed(6);
    randomTest(172, 181, false); // Low
    mt.seed(7);
    randomTest(200, 200, true); // Low
    mt.seed(8);
    randomTest(222, 222, false); // Low
    mt.seed(9);
    randomTest(250, 250, true); // Low
    mt.seed(10);
    randomTest(250, 250, false); // Low

    // Large constraints, single meeting point
    mt.seed(10007);
    randomTest(266, 300); // Single
    mt.seed(20019);
    randomTest(333, 333); // Single
    mt.seed(30012);
    randomTest(398, 329); // Single
    mt.seed(40003);
    randomTest(420, 420); // Single
    mt.seed(50018);
    randomTest(444, 444); // Single
    mt.seed(60001);
    randomTest(450, 450); // Single
    mt.seed(70010);
    randomTest(483, 471); // Single
    mt.seed(80037);
    randomTest(500, 500); // Single
    mt.seed(90024);
    randomTest(500, 500); // Single
    mt.seed(100048);
    randomTest(500, 500); // Single

    // Large constraints and no single cell
    sampleInput4(); // None

    mt.seed(1000000);
    randomTest(300, 300); // None
    mt.seed(2000000);
    randomTest(333, 333); // None
    mt.seed(3000043);
    randomTest(366, 366); // None
    mt.seed(4000015);
    randomTest(400, 400); // None
    mt.seed(5000000);
    randomTest(420, 420); // None
    mt.seed(6000000);
    randomTest(433, 433, true); // None
    mt.seed(7000000);
    randomTest(444, 444, true); // None
    mt.seed(8000000);
    randomTest(466, 333, true); // None
    mt.seed(9000000);
    randomTest(321, 420, true); // None

    maxTest1(); // None
    maxTest2(); // None
    maxTest3(); // None
    maxTest4(); // None
    maxTest5(); // None
    maxTest6(); // None
    maxTest7(); // None
    maxTest8(); // None
    maxTest9(); // None
    maxTest10(); // None

    printTests();
    return 0;
}
